'use strict';

var winston = require('winston');
// https://github.com/winstonjs/winston

var _require = require('q-mod-date'),
    newDate = _require.newDate;

var _require2 = require('./helpers'),
    getArrOfMethods = _require2.getArrOfMethods,
    addLevelMethodsBasedOnWinstonInstance = _require2.addLevelMethodsBasedOnWinstonInstance,
    padNr = _require2.padNr,
    deletePreviousLogFile = _require2.deletePreviousLogFile,
    padRightLvls = _require2.padRightLvls;

var JsonMemory = require('./jsonMemoryTransport');

///////////////// CUSTOM TIME STAMPS
function returnCustomTimeStampType() {
  var timeStampType = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'd-compact';

  // d- defoult long compact compressed old
  // t- defoult ordinary long compressed
  function getTimeStamp(type) {
    return function customTimeStamp() {
      var date = newDate(type);
      // const date = new Date().toISOString();
      // console.log('date', date);
      return date;
    };
  }

  if (timeStampType === 'd-compact') {
    return getTimeStamp('compact');
  } else if (timeStampType === 'd-defoult') {
    return getTimeStamp('defoult');
  } else if (timeStampType === 'd-long') {
    return getTimeStamp('long');
  } else if (timeStampType === 'd-compressed') {
    return getTimeStamp('compressed');
  } else if (timeStampType === 'd-old') {
    return getTimeStamp('old');
  } else if (timeStampType === 't-defoult') {
    return getTimeStamp('time-default');
  } else if (timeStampType === 't-ordinary') {
    return getTimeStamp('time-ordinary');
  } else if (timeStampType === 't-long') {
    return getTimeStamp('time-long');
  } else if (timeStampType === 't-compressed') {
    return getTimeStamp('time-compressed');
  } else {
    return getTimeStamp('compact');
  }
}

///////////////// CUSTOM FORMATTER
function serviceLogFormatter() {
  // oryginal timestamp: options.timestamp()
  return function customFormatter(options) {
    var logMsgConstructor = '';
    if (options.timestamp) logMsgConstructor += '| ' + options.timestamp() + ' ';
    logMsgConstructor += '' + padRightLvls(options.level, '\u2002');
    if (options.message) logMsgConstructor += '| ' + options.message + ' ';
    // dla console transportu dodać: += '\n\t'+ JSON.stringify(options.meta)
    if (options.meta && Object.keys(options.meta).length) logMsgConstructor += ' ' + JSON.stringify(options.meta);
    return logMsgConstructor;
  };
}

function returnCustomFormatterType() {
  var formatterType = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'service';

  if (formatterType === 'service') {
    return serviceLogFormatter();
  } else {
    return serviceLogFormatter();
  }
}

///////////////// CUSTOM TRANSPORSTS
function addNewTransport() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$name = _ref.name,
      name = _ref$name === undefined ? '' : _ref$name,
      _ref$fileName = _ref.fileName,
      fileName = _ref$fileName === undefined ? '.logs.log' : _ref$fileName,
      _ref$options = _ref.options,
      options = _ref$options === undefined ? {} : _ref$options;

  // opts for file transport
  var _options$newLogFilePe = options.newLogFilePerInstance,
      newLogFilePerInstance = _options$newLogFilePe === undefined ? true : _options$newLogFilePe,
      _options$addTimeStamp = options.addTimeStamp,
      addTimeStamp = _options$addTimeStamp === undefined ? true : _options$addTimeStamp,
      _options$timeStampTyp = options.timeStampType,
      timeStampType = _options$timeStampTyp === undefined ? 'd-compact' : _options$timeStampTyp,
      _options$formatterTyp = options.formatterType,
      formatterType = _options$formatterTyp === undefined ? 'service' : _options$formatterTyp,
      _options$colorize = options.colorize,
      colorize = _options$colorize === undefined ? false : _options$colorize,
      _options$maxsize = options.maxsize,
      maxsize = _options$maxsize === undefined ? null : _options$maxsize,
      _options$maxFiles = options.maxFiles,
      maxFiles = _options$maxFiles === undefined ? null : _options$maxFiles,
      _options$zippedArchiv = options.zippedArchive,
      zippedArchive = _options$zippedArchiv === undefined ? false : _options$zippedArchiv,
      _options$level = options.level,
      level = _options$level === undefined ? 'silly' : _options$level,
      _options$json = options.json,
      json = _options$json === undefined ? false : _options$json;

  if (name === 'file') {
    newLogFilePerInstance && deletePreviousLogFile(fileName);
    this.winstonLogger.add(winston.transports.File, {
      colorize: colorize,
      json: json,
      timestamp: addTimeStamp ? returnCustomTimeStampType(timeStampType) : false,
      formatter: returnCustomFormatterType(formatterType),
      filename: fileName,
      maxsize: maxsize,
      maxFiles: maxFiles,
      zippedArchive: zippedArchive,
      level: level });
    return this;
  } else {
    return this;
  }
}

///////////////////////////////////////// MAIN  /////////////////
var loggerFns = {
  ctx: function ctx() {
    var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    this.ctxStep = 0; //stepper
    if (typeof str !== 'string') {
      throw new Error('q-mod-logger - ctx() - not passed string to the fn.');
    }
    this.ctxVar = str + ' - ';
    return this;
  },
  fnName: function fnName() {
    var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    if (typeof str !== 'string') {
      throw new Error('q-mod-logger - fnName() - not passed string to the fn.');
    }
    this.fnNameVar = str + '() - ';
    return this;
  },
  resetFnName: function resetFnName() {
    this.fnNameVar = '';
    return this;
  },
  resetCtx: function resetCtx() {
    this.ctxVar = '';
    this.ctxStep = 0; //stepper
    return this;
  },


  // predefined log msgs
  txtCalledFn: function txtCalledFn(fnName) {
    var lvlName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'verbose';

    if (fnName) this.fnName(fnName);
    if (typeof this[lvlName] !== 'function') {
      throw new Error('q-mod-logger - txtCalledFn() - do not found that level method ' + fnName + ' in the instace of this logger!');
    }
    this[lvlName]('Called this fn.');
    return this;
  },
  txtState: function txtState(varName, state) {
    this.debug('State of variable --> ' + varName + ':', state);
    return this;
  },
  txtArgs: function txtArgs(state) {
    this.debug('Passed args to the fn:', state);
    return this;
  },


  // exports
  exportToVars: function exportToVars() {
    var errMsgs = this.winstonLogger.transports.memory.errorOutput;
    var writeMsgs = this.winstonLogger.transports.memory.writeOutput;
    var arrOfMsgs = [errMsgs, writeMsgs];
    // concating but not sorted!!
    this.exportedAll = [].concat.apply([], arrOfMsgs);
    this.exportedDebug = errMsgs;
    this.exportedJsonAll = this.winstonLogger.transports.jsonMemory.trOutput;
    this.exportedJsonInterface = this.winstonLogger.transports.jsonMemory.trOutputForInterface;
    return this;
  },
  export: function _export() {
    var what = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'all';

    this.exportToVars();
    if (what === 'all') {
      return this.exportedAll;
    } else if (what === 'debug') {
      return this.exportedDebug;
    } else {
      return this;
    }
  },
  exportJson: function exportJson() {
    var what = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'all';

    this.exportToVars();
    if (what === 'all') {
      return this.exportedJsonAll;
    } else if (what === 'interface') {
      return this.exportedJsonInterface;
    } else {
      return this;
    }
  },


  // tools
  addNewTransport: addNewTransport,

  // helper - pad stepper nr with 0ros
  padNr: padNr
};

var loggerVars = {
  winstonLogger: undefined,
  fnNameVar: '',
  ctxVar: '',
  ctxStep: 0,
  exportedAll: undefined,
  exportedDebug: undefined
};

function logger() {
  var opt = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};


  // opts
  // difoltowy: service logger - log all lvls.
  var _opt$addTimeStamp = opt.addTimeStamp,
      addTimeStamp = _opt$addTimeStamp === undefined ? true : _opt$addTimeStamp,
      _opt$timeStampType = opt.timeStampType,
      timeStampType = _opt$timeStampType === undefined ? 'd-compact' : _opt$timeStampType,
      _opt$formatterType = opt.formatterType,
      formatterType = _opt$formatterType === undefined ? 'service' : _opt$formatterType,
      _opt$colorize = opt.colorize,
      colorize = _opt$colorize === undefined ? false : _opt$colorize,
      _opt$level = opt.level,
      level = _opt$level === undefined ? 'silly' : _opt$level,
      _opt$json = opt.json,
      json = _opt$json === undefined ? false : _opt$json;

  // init

  var winstonLogger = new winston.Logger({
    transports: [new winston.transports.Memory({
      colorize: colorize,
      level: level,
      json: json,
      timestamp: addTimeStamp ? returnCustomTimeStampType(timeStampType) : false,
      formatter: returnCustomFormatterType(formatterType)
    })]
  });

  // JsonMemory Transport
  winstonLogger.add(JsonMemory, {
    colorize: colorize,
    level: level,
    timestamp: addTimeStamp ? returnCustomTimeStampType('d-defoult') : false
  });

  var getLevelsObj = winstonLogger.levels;
  var arrOfLevelKeys = getArrOfMethods(getLevelsObj);
  addLevelMethodsBasedOnWinstonInstance(arrOfLevelKeys, loggerVars);
  loggerVars.winstonLogger = winstonLogger;

  // compose
  var composeLoggerObj = Object.assign({}, loggerVars, loggerFns);
  return composeLoggerObj;
}

// module API::
module.exports = {
  logger: logger
};