'use strict';

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = require('util');
var winston = require('winston');
var cycle = require('cycle');
var colors = require('colors/safe');

// my jsonMemory transport

exports = module.exports = winston.transports.jsonMemory = function (options) {
  this.name = 'jsonMemory';
  this.level = options.level || 'info';
  this.json = true;
  this.colorize = options.colorize || false;
  this.prettyPrint = options.prettyPrint || false;
  this.timestamp = typeof options.timestamp !== 'undefined' ? options.timestamp : false;
  this.showLevel = options.showLevel === undefined ? true : options.showLevel;
  this.label = options.label || null;
  this.logstash = options.logstash || false;
  this.depth = options.depth || null;
  this.align = options.align || false;

  this.trOutput = [];
  this.trOutputForInterface = [];

  if (this.json) {
    this.stringify = options.stringify || function (obj) {
      return JSON.stringify(obj, null, 2);
    };
  }
};

var jsonMemory = winston.transports.jsonMemory;

util.inherits(jsonMemory, winston.Transport);

jsonMemory.prototype.log = function (level, msg, meta, callback) {
  if (this.silent) {
    return callback && callback(null, true);
  }
  var self = this;
  var output;

  output = commonlog({
    colorize: this.colorize,
    json: this.json,
    level: level,
    message: msg,
    meta: meta,
    stringify: this.stringify,
    timestamp: this.timestamp,
    showLevel: this.showLevel,
    prettyPrint: this.prettyPrint,
    raw: this.raw,
    label: this.label,
    logstash: this.logstash,
    depth: this.depth,
    formatter: this.formatter,
    align: this.align,
    humanReadableUnhandledException: this.humanReadableUnhandledException
  });

  // code embrittlement due to default winston log levels
  // watchout when implementing custom log levels
  var parsedOutput = JSON.parse(output);
  if (level === 'error' || level === 'warn' || level === 'info') {
    this.trOutputForInterface.push(parsedOutput);
    this.trOutput.push(parsedOutput);
  } else {
    this.trOutput.push(parsedOutput);
  }

  self.emit('logged');
  if (callback) callback(null, true);
};

///////////////////////////////////////// FNS form WINSTON /////////////////


//
// ### function log (options)
// #### @options {Object} All information about the log serialization.
// Generic logging function for returning timestamped strings
// with the following options:
//
//    {
//      level:     'level to add to serialized message',
//      message:   'message to serialize',
//      meta:      'additional logging metadata to serialize',
//      colorize:  false, // Colorizes output (only if `.json` is false)
//      align:     false  // Align message level.
//      timestamp: true   // Adds a timestamp to the serialized message
//      label:     'label to prepend the message'
//    }
//
var commonlog = function commonlog(options) {
  var timestampFn = typeof options.timestamp === 'function' ? options.timestamp : exports.timestamp,
      timestamp = options.timestamp ? timestampFn() : null,
      showLevel = options.showLevel === undefined ? true : options.showLevel,
      meta = options.meta !== null && options.meta !== undefined && !(options.meta instanceof Error) ? exportsclone(cycle.decycle(options.meta)) : options.meta || null,
      output;

  //
  // raw mode is intended for outputing winston as streaming JSON to STDOUT
  //
  if (options.raw) {
    if ((typeof meta === 'undefined' ? 'undefined' : (0, _typeof3.default)(meta)) !== 'object' && meta != null) {
      meta = { meta: meta };
    }
    output = exportsclone(meta) || {};
    output.level = options.level;
    //
    // Remark (jcrugzz): This used to be output.message = options.message.stripColors.
    // I do not know why this is, it does not make sense but im handling that
    // case here as well as handling the case that does make sense which is to
    // make the `output.message = options.message`
    //
    output.message = options.message.stripColors ? options.message.stripColors : options.message;

    return JSON.stringify(output);
  }

  //
  // json mode is intended for pretty printing multi-line json to the terminal
  //
  if (options.json || true === options.logstash) {
    if ((typeof meta === 'undefined' ? 'undefined' : (0, _typeof3.default)(meta)) !== 'object' && meta != null) {
      meta = { meta: meta };
    }

    output = exportsclone(meta) || {};
    output.level = options.level;
    output.message = output.message || '';

    if (options.label) {
      output.label = options.label;
    }
    if (options.message) {
      output.message = options.message;
    }
    if (timestamp) {
      output.timestamp = timestamp;
    }

    if (options.logstash === true) {
      // use logstash format
      var logstashOutput = {};
      if (output.message !== undefined) {
        logstashOutput['@message'] = output.message;
        delete output.message;
      }

      if (output.timestamp !== undefined) {
        logstashOutput['@timestamp'] = output.timestamp;
        delete output.timestamp;
      }

      logstashOutput['@fields'] = exportsclone(output);
      output = logstashOutput;
    }

    if (typeof options.stringify === 'function') {
      return options.stringify(output);
    }

    return JSON.stringify(output, function (key, value) {
      return value instanceof Buffer ? value.toString('base64') : value;
    });
  }

  //
  // Remark: this should really be a call to `util.format`.
  //
  if (typeof options.formatter == 'function') {
    options.meta = meta;
    return String(options.formatter(exportsclone(options)));
  }

  output = timestamp ? timestamp + ' - ' : '';
  if (showLevel) {
    output += options.colorize === 'all' || options.colorize === 'level' || options.colorize === true ? configcolorize(options.level) : options.level;
  }

  output += options.align ? '\t' : '';
  output += timestamp || showLevel ? ': ' : '';
  output += options.label ? '[' + options.label + '] ' : '';
  output += options.colorize === 'all' || options.colorize === 'message' ? configcolorize(options.level, options.message) : options.message;

  if (meta !== null && meta !== undefined) {
    if (meta && meta instanceof Error && meta.stack) {
      meta = meta.stack;
    }

    if ((typeof meta === 'undefined' ? 'undefined' : (0, _typeof3.default)(meta)) !== 'object') {
      output += ' ' + meta;
    } else if (Object.keys(meta).length > 0) {
      if (typeof options.prettyPrint === 'function') {
        output += ' ' + options.prettyPrint(meta);
      } else if (options.prettyPrint) {
        output += ' ' + '\n' + util.inspect(meta, false, options.depth || null, options.colorize);
      } else if (options.humanReadableUnhandledException && Object.keys(meta).length === 5 && meta.hasOwnProperty('date') && meta.hasOwnProperty('process') && meta.hasOwnProperty('os') && meta.hasOwnProperty('trace') && meta.hasOwnProperty('stack')) {

        //
        // If meta carries unhandled exception data serialize the stack nicely
        //
        var stack = meta.stack;
        delete meta.stack;
        delete meta.trace;
        output += ' ' + exportsserialize(meta);

        if (stack) {
          output += '\n' + stack.join('\n');
        }
      } else {
        output += ' ' + exportsserialize(meta);
      }
    }
  }

  return output;
};

// Fix colors not appearing in non-tty environments
colors.enabled = true;

var allColors = {};

var configcolorize = function configcolorize(level, message) {
  if (typeof message === 'undefined') message = level;

  var colorized = message;
  if (allColors[level] instanceof Array) {
    for (var i = 0, l = allColors[level].length; i < l; ++i) {
      colorized = colors[allColors[level][i]](colorized);
    }
  } else if (allColors[level].match(/\s/)) {
    var colorArr = allColors[level].split(/\s+/);
    for (var ii = 0; ii < colorArr.length; ++ii) {
      colorized = colors[colorArr[ii]](colorized);
    }
    allColors[level] = colorArr;
  } else {
    colorized = colors[allColors[level]](colorized);
  }

  return colorized;
};

var exportsclone = function exportsclone(obj) {
  //
  // We only need to clone reference types (Object)
  //
  var copy = {};

  if (obj instanceof Error) {
    // With potential custom Error objects, this might not be exactly correct,
    // but probably close-enough for purposes of this lib.
    copy = { message: obj.message };
    Object.getOwnPropertyNames(obj).forEach(function (key) {
      copy[key] = obj[key];
    });

    return copy;
  } else if (!(obj instanceof Object)) {
    return obj;
  } else if (obj instanceof Date) {
    return new Date(obj.getTime());
  }

  for (var i in obj) {
    if (Array.isArray(obj[i])) {
      copy[i] = obj[i].slice(0);
    } else if (obj[i] instanceof Buffer) {
      copy[i] = obj[i].slice(0);
    } else if (typeof obj[i] != 'function') {
      copy[i] = obj[i] instanceof Object ? exportsclone(obj[i]) : obj[i];
    } else if (typeof obj[i] === 'function') {
      copy[i] = obj[i];
    }
  }

  return copy;
};

//
// ### function serialize (obj, key)
// #### @obj {Object|literal} Object to serialize
// #### @key {string} **Optional** Optional key represented by obj in a larger object
// Performs simple comma-separated, `key=value` serialization for Loggly when
// logging to non-JSON inputs.
//
var exportsserialize = function exportsserialize(obj, key) {
  // symbols cannot be directly casted to strings
  if ((typeof key === 'undefined' ? 'undefined' : (0, _typeof3.default)(key)) === 'symbol') {
    key = key.toString();
  }
  if ((typeof obj === 'undefined' ? 'undefined' : (0, _typeof3.default)(obj)) === 'symbol') {
    obj = obj.toString();
  }

  if (obj === null) {
    obj = 'null';
  } else if (obj === undefined) {
    obj = 'undefined';
  } else if (obj === false) {
    obj = 'false';
  }

  if ((typeof obj === 'undefined' ? 'undefined' : (0, _typeof3.default)(obj)) !== 'object') {
    return key ? key + '=' + obj : obj;
  }

  if (obj instanceof Buffer) {
    return key ? key + '=' + obj.toString('base64') : obj.toString('base64');
  }

  var msg = '',
      keys = Object.keys(obj),
      length = keys.length;

  for (var i = 0; i < length; i++) {
    if (Array.isArray(obj[keys[i]])) {
      msg += keys[i] + '=[';

      for (var j = 0, l = obj[keys[i]].length; j < l; j++) {
        msg += exportsserialize(obj[keys[i]][j]);
        if (j < l - 1) {
          msg += ', ';
        }
      }

      msg += ']';
    } else if (obj[keys[i]] instanceof Date) {
      msg += keys[i] + '=' + obj[keys[i]];
    } else {
      msg += exportsserialize(obj[keys[i]], keys[i]);
    }

    if (i < length - 1) {
      msg += ', ';
    }
  }

  return msg;
};